# -*- encoding: utf-8 -*-
require 'spec_helper'
#作家

describe AuthorsController do
  before do
    Factory :admin
    @user = Factory( :user_yas)
    @author = @user.author
    @artist = Factory :artist_yas, :author_id => @author.id
  end

  describe '一覧表示に於いて' do
    before do
      Author.stub(:list).and_return([@author, @author, @author])
      sign_in @user
    end
    context '事前チェックする' do
      it '与えられたpageがセットされている' do
        get :index, :page => 5
        assigns(:page).should eq 5
      end
      it '省略されると@pageに1値が入る' do
        get :index
        assigns(:page).should eq 1
      end
      it '与えられたpage_sizeがセットされている' do
        get :index, :page_size => 15
        assigns(:page_size).should eq 15
      end
      it '省略されると@page_sizeにデフォルト値が入る' do
        get :index
        assigns(:page_size).should eq Author.default_page_size
      end
      it '最大を超えると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 1500
        assigns(:page_size).should eq Author.max_page_size
      end
      it '不正な値が入ると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 0
        assigns(:page_size).should eq Author.default_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
      it '作家モデルに一覧を問い合わせている' do
        Author.should_receive(:list).exactly(1)
        get :index
      end
      it '@authorsにリストを取得している' do
        get :index
        assigns(:authors).should have_at_least(3).items
      end
      context 'html形式' do
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :index, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがリスト構造になっている' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.should have_at_least(3).items
        end
        it 'リストの先頭くらいは作家っぽいものであって欲しい' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.first.has_key?("name").should be_true
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :index
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :index
          response.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :index, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :index, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end
  
  describe '閲覧に於いて' do
    before do
      Author.stub(:show).and_return(@author)
      sign_in @user
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :show, :id => @author.id
        response.should be_success
      end
      it '作家モデルに単体取得を問い合わせている' do
        Author.should_receive(:show).exactly(1)
        get :show
      end
      it '@authorにアレを取得している' do
        get :show, :id => @author.id
        assigns(:author).should eq(@author)
      end
      context 'html形式' do
        it 'showテンプレートを描画する' do
          get :show, :id => @author.id
          response.should render_template("show")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :show, :id => @author.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがアレになっている' do
          get :show, :id => @author.id, :format => :json
          json = JSON.parse response.body
          json["name"].should match(/yas/)
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :show, :id => @author.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :show, :id => @author.id
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :show, :id => @author.id, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :show, :id => @author.id, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
=begin
    context '対象作家がないとき' do
      context 'html形式' do
        it '例外404 not_foundを返す' do
          lambda{
            get :show, :id => 0
          }.should raise_error(ActiveRecord::RecordNotFound)
        end
      end
      context 'json形式' do
        it '例外404 not_foundを返す' do
          lambda{ 
            get :show, :id => 0, :format => :json
          }.should raise_error(ActiveRecord::RecordNotFound)
        end
      end
    end
=end
  end
  
  describe '作家数取得に於いて' do
    before do
      Author.should_receive(:visible_count).and_return(3)
#      sign_in @user
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :count, :format => :json
        response.should be_success 
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :count, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがHash構造になっていてコミック数が1である' do
          get :count, :format => :json
          json = JSON.parse response.body
          json["count"].should == 3
        end
      end
    end
  end

  describe 'コミック一覧表示に於いて' do
    #指定作家のコミックpage_size件を一覧形式で表示する。ただし、作者によって非公開にされたコミックは含まない。 
    before do
      @comic = Factory :normal_comic, :author_id => @author.id
      Comic.stub(:list_by_author).and_return([@comic, @comic, @comic])
      sign_in @user
    end
    context '事前チェックする' do
      it '与えられたpageがセットされている' do
        get :comics, :id => @author.id, :page => 5
        assigns(:page).should eq 5
      end
      it '省略されると@pageに1値が入る' do
        get :comics, :id => @author.id
        assigns(:page).should eq 1
      end
      it '与えられたpage_sizeがセットされている' do
        get :comics, :id => @author.id, :page_size => 15
        assigns(:page_size).should eq 15
      end
      it '省略されると@page_sizeにデフォルト値が入る' do
        get :comics, :id => @author.id
        assigns(:page_size).should eq Author.default_page_size
      end
      it '最大を超えると@page_sizeにデフォルト最大値が入る' do
        get :comics, :id => @author.id, :page_size => 1500
        assigns(:page_size).should eq Author.max_page_size
      end
      it '不正な値が入ると@page_sizeにデフォルト最大値が入る' do
        get :comics, :id => @author.id, :page_size => 0
        assigns(:page_size).should eq Author.default_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :comics, :id => @author.id
        response.should be_success 
      end
      it 'コミックモデルに一覧を問い合わせている' do
        Comic.should_receive(:list_by_author).exactly(1)
        get :comics, :id => @author.id
      end
      it '@comicsにリストを取得している' do
        get :comics, :id => @author.id
        assigns(:comics).should have_at_least(3).items
      end
      context 'html形式' do
        it 'comicsテンプレートを描画する' do
          get :comics, :id => @author.id
          response.should render_template("comics")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :comics, :id => @author.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがリスト構造になっている' do
          get :comics, :id => @author.id, :format => :json
          json = JSON.parse response.body
          json.should have_at_least(3).items
        end
        it 'リストの先頭くらいはコミックっぽいものであって欲しい' do
          get :comics, :id => @author.id, :format => :json
          json = JSON.parse response.body
          json.first.has_key?("title").should be_true
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :comics, :id => @author.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :comics, :id => @author.id
          response.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :comics, :id => @author.id, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :comics, :id => @author.id, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end
  
  describe 'コマ一覧表示に於いて' do
    #指定作家のコマpage_size件を一覧形式で表示する。ただし、作者によって非公開にされたコミックのコマは含まない。 
    before do
      @comic = Factory :normal_comic, :author_id => @author.id
      @panel = Factory :panel, :author_id => @author.id, :comic_id => @comic.id
      Panel.stub(:list_by_author).and_return([@panel, @panel, @panel])
      sign_in @user
    end
    context '事前チェックする' do
      it '与えられたpageがセットされている' do
        get :panels, :id => @author.id, :page => 5
        assigns(:page).should eq 5
      end
      it '省略されると@pageに1値が入る' do
        get :panels, :id => @author.id
        assigns(:page).should eq 1
      end
      it '与えられたpage_sizeがセットされている' do
        get :panels, :id => @author.id, :page_size => 15
        assigns(:page_size).should eq 15
      end
      it '省略されると@page_sizeにデフォルト値が入る' do
        get :panels, :id => @author.id
        assigns(:page_size).should eq Author.default_page_size
      end
      it '最大を超えると@page_sizeにデフォルト最大値が入る' do
        get :panels, :id => @author.id, :page_size => 1500
        assigns(:page_size).should eq Author.max_page_size
      end
      it '不正な値が入ると@page_sizeにデフォルト最大値が入る' do
        get :panels, :id => @author.id, :page_size => 0
        assigns(:page_size).should eq Author.default_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :panels, :id => @author.id
        response.should be_success 
      end
      it 'コマモデルに一覧を問い合わせている' do
        Panel.should_receive(:list_by_author).exactly(1)
        get :panels, :id => @author.id
      end
      it '@panelsにリストを取得している' do
        get :panels, :id => @author.id
        assigns(:panels).should have_at_least(3).items
      end
      context 'html形式' do
        it 'panelsテンプレートを描画する' do
          get :panels, :id => @author.id
          response.should render_template("panels")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :panels, :id => @author.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがリスト構造になっている' do
          get :panels, :id => @author.id, :format => :json
          json = JSON.parse response.body
          json.should have_at_least(3).items
        end
        it 'リストの先頭くらいはコマっぽいものであって欲しい' do
          get :panels, :id => @author.id, :format => :json
          json = JSON.parse response.body
          json.first.has_key?("link").should be_true
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :panels, :id => @author.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :panels, :id => @author.id
          response.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :panels, :id => @author.id, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :panels, :id => @author.id, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end
end
